%% Convolution Neural Network Exercise
clc
clear 
close all
% diary on;
% diary('run_log');

%% STEP 0: Initialize Parameters and Load Data
rng(1);

%  complete the config.m to config the network structure;
cnnConfig = config2();
%  calling cnnInitParams() to initialize parameters
[theta, meta] = cnnInitParams(cnnConfig);

% Load MNIST Data
images = loadMNISTImages('../train-images-idx3-ubyte');
d = cnnConfig.layer{1}.dimension;
images = reshape(images,d(1),d(2),d(3),[]);
labels = loadMNISTLabels('../train-labels-idx1-ubyte');
labels(labels==0) = 10; % Remap 0 to 10

%%======================================================================
%% STEP 3: Learn Parameters
%  Implement minFuncSGD.m, then train the model.
rng(1);

options.epochs    = 3;
options.minibatch = 128;
options.alpha     = .01; % 0.1
options.momentum  = .95;

tic
opttheta = minFuncSGD(@(x,y,z)cnnCost(x, y, z, cnnConfig, meta), theta, images, labels, options);
toc
theta = opttheta; % Save for backup

save('cnn_workspace.mat');

%%======================================================================
%% STEP 4: Test
%  Test the performance of the trained model using the MNIST test set. Your
%  accuracy should be above 97% after 3 epochs of training

load('cnn_workspace.mat');

nrtest = 10000; % upto 10000
subidx = randsample(10000, nrtest);
testImages = loadMNISTImages('../t10k-images-idx3-ubyte');
testImages = testImages(:, subidx);
testImagesr = reshape(testImages, d(1), d(2), d(3),[]);
testLabels = loadMNISTLabels('../t10k-labels-idx1-ubyte');
testLabelsr = testLabels(subidx);
testLabelsr(testLabelsr==0) = 10; % Remap 0 to 10

tic
[~, ~, preds] = cnnCost(opttheta, testImagesr, [], cnnConfig, meta, true);
toc
acc = sum(preds==testLabelsr)/length(preds);
fprintf('Accuracy is %f\n',acc);

% diary off;

%% Check wrong classifications
temp = preds==testLabelsr;
wrong_idx = find(temp == 0);
for ii = 1:length(wrong_idx)
    i = wrong_idx(ii);
    img = testImages(:, i);
    img = reshape(img, 28, 28);
    
    figure(1); clf;
    imagesc(img); colormap gray;
    title(sprintf('[%d/%d] Correct label: %d / Predicted label: %d' ...
        , ii, length(wrong_idx), testLabelsr(i), preds(i)));
    drawnow;
    pause();
end

%% Visualize Filter
thetastack = thetaChange(theta, meta, 'vec2stack', cnnConfig);

W = thetastack{2}.W;
Wstacked = zeros(81, 20);
for i = 1:20
    cW = W(:, :, 1, i);
    Wstacked(:, i) = cW(:);
end
figure(3); clf;
display_network(Wstacked);
title('Convolution Filters');




